/*
 *  $Id$
 */
 
/**
 *  @file btchat_backend.c
 *
 *  @project btChat (Collin's btChat) (http://www.mulliner.org/bluetooth/btchat/)
 *
 *  @author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 *  backend stuff
 *
 */

#include "internal.h"

#include "account.h"
#include "accountopt.h"
#include "debug.h"
//#include "multi.h"
#include "notify.h"
#include "proxy.h"
#include "prpl.h"
#include "server.h"
#include "util.h"

#include "btchat_util.h"
#include "btchat_client_protocol.h"
#include "btchat.h"

/**
 *  @brief send login to btchatd
 *
 *  @param gc pointer to gaimconnection
 *
 *  @returns 1
 */
int handle_event_login(GaimConnection *gc)
{
	GaimAccount *account = gaim_connection_get_account(gc);
	struct btchat_globals_t *btg = gc->proto_data;
	unsigned char out_buffer[1024];
	struct btcclient_protocol_header_t *he = (struct btcclient_protocol_header_t*) out_buffer;
	struct btcclient_protocol_login_t *login = (struct btcclient_protocol_login_t*) (out_buffer + sizeof(struct btcclient_protocol_header_t));
	

	memset(out_buffer, 0, sizeof(out_buffer));

	he->version = 0x01;
	he->pkg_type = BTCCLIENT_PROTOCOL_LOGIN;
	he->time = time(NULL);
	he->length = sizeof(struct btcclient_protocol_header_t) + sizeof(struct btcclient_protocol_login_t);

	memcpy(login->name, account->username, strlen(account->username) < sizeof(login->name) ? strlen(account->username) : sizeof(login->name));
	
	nwrite(btg->socket, out_buffer, he->length);
	
	return(1);
}

/**
 *  @brief send a message
 *
 *  @param gc    pointer to the gaimconnection
 *  @param who   destination
 *  @param msg   message
 *  @param flags message flags
 *
 *  @returns 1
 */
int handle_event_msg_send(GaimConnection *gc, const char *who, const char *msg, GaimConvImFlags flags)
{
	GaimAccount *account = gaim_connection_get_account(gc);
	struct btchat_globals_t *btg = gc->proto_data;
	unsigned char out_buffer[0xffff];
	struct btcclient_protocol_header_t *he = (struct btcclient_protocol_header_t*) out_buffer;
	struct btcclient_protocol_msg_t *msghe = (struct btcclient_protocol_msg_t*) (out_buffer + sizeof(struct btcclient_protocol_header_t));
	GaimBuddy *b;
	GaimGroup *g;
	struct btchat_buddy_info_t *binfo;
	const char *domain = {"btChat\0"};


	g = gaim_find_group(domain);
	if (g == NULL) {
		gaim_debug(GAIM_DEBUG_INFO, "btchat", "handle_event_msg_send(): group not found!\n");
		return(-5);
	}

	b = gaim_find_buddy_in_group(account, who, g);
	if (b == NULL) {
		gaim_debug(GAIM_DEBUG_INFO, "btchat", "handle_event_msg_send(): buddy not found!\n");
		return(-2);
	}
	else {
		binfo = (struct btchat_buddy_info_t*) b->proto_data;
		if (binfo == NULL) {
			gaim_debug(GAIM_DEBUG_INFO, "btchat", "handle_event_msg_send(): buddy proto_data not found!\n");
			return(-3);
		}
		else {
			/* no connection to this client */
			if (binfo->node.connection == BTCHAT_CONNECTION_TYPE_NOTCONNECTED) {
				gaim_debug(GAIM_DEBUG_INFO, "btchat", "handle_event_msg_send(): no path to buddy!\n");
				return(-4);
			}
		}
	}

	if (strlen(msg) > (0xffff - sizeof(struct btcclient_protocol_msg_t) - sizeof(struct btcclient_protocol_header_t))) {
		gaim_debug(GAIM_DEBUG_INFO, "btchat", "handle_event_msg_send(): message to big!\n");
		return(-1);
	}
	
	gaim_debug(GAIM_DEBUG_INFO, "btchat", "handle_event_msg_send(): everything ok!\n");
	
	memset(out_buffer, 0, sizeof(out_buffer));
	
	he->version = 0x01;
	he->pkg_type = BTCCLIENT_PROTOCOL_MSG;
	he->time = time(NULL);

	gaim_debug(GAIM_DEBUG_INFO, "btchat", "handle_event_msg_send(): who = %s!\n", who);
	
	str2ba(who, msghe->dst);
	/* src will be set by btchatd */
	memcpy(out_buffer + sizeof(struct btcclient_protocol_msg_t) + sizeof(struct btcclient_protocol_header_t), msg, strlen(msg));

	he->length = sizeof(struct btcclient_protocol_msg_t) + sizeof(struct btcclient_protocol_header_t) + strlen(msg);
	
	nwrite(btg->socket, out_buffer, he->length);
	
	return(1);
}

/**
 *  @brief handle incoming message
 *
 *  @param gc pointer to GAIM connection
 *  @param e  pointer to btchat_event
 *
 *  @returns 1
 */
int handle_event_msg_received(GaimConnection *gc, struct btchat_event_t *e)
{
	char bd_addr[20];
	
	
	ba2str(e->src.addr, bd_addr);
	serv_got_im(gc, bd_addr, e->data.msg.msg, 0, e->when);
	
	return(1);
}

/**
 *  @brief process node (update/new) events
 *
 *  @param gc pointer to GAIM connection
 *  @param e  pointer to btchat_event
 *
 *  @returns 1
 */
int handle_event_node(GaimConnection *gc, struct btchat_event_t *e)
{
	GaimAccount *account = gaim_connection_get_account(gc);
	GaimBuddy *b;
	GaimGroup *g;
	const char *domain = {"btChat\0"};
	struct btchat_buddy_info_t *binfo;
	char bd_addr[20];
	struct btcclient_protocol_account_t *node;
	guint status;

	
	/* get node pointer */
	node = &e->src;

	g = gaim_find_group(domain);
	if (g == NULL) {
		g = gaim_group_new(domain);
		gaim_blist_add_group(g, NULL);
	}

	memset(bd_addr, 0, sizeof(bd_addr));
	ba2str(node->addr, bd_addr);
	b = gaim_find_buddy_in_group(account, bd_addr, g);

	/* existing */
	if (b != NULL) {
		if (node->state == BTCHAT_NODE_STATE_OFFLINE) {
			status = UC_UNAVAILABLE;
			//printf("buddy offline: %d\n", status);
			serv_got_update(gc, b->name, 0, 0, 0, 0, status);
			//printf("buddy offline2: %d\n", b->uc);
			
			/* remove buddy */
		/*	if (b->proto_data != NULL) {
				free(b->proto_data);
				b->proto_data = NULL;
			}
			gaim_blist_remove_buddy(b);
		*/
		}
		else {
			status = 2 | (node->state<<5);
			//printf("buddy update: %d\n", status);
			serv_got_update(gc, b->name, 1, 0, 0, 0, status);
			serv_got_alias(gc, b->name, node->name);
			//printf("buddy update2: %d\n", b->uc);
			
			if (b->proto_data == NULL) {
				b->proto_data = malloc(sizeof(struct btchat_buddy_info_t));
			}
			binfo = (struct btchat_buddy_info_t*) b->proto_data;
			memcpy(&binfo->node, node, sizeof(struct btcclient_protocol_account_t));
		}
	}
	/* new */
	else {
		b = gaim_buddy_new(account, bd_addr, NULL);
		gaim_blist_add_buddy(b, NULL, g, NULL);

		binfo = malloc(sizeof(struct btchat_buddy_info_t));
		memset(binfo, 0, sizeof(struct btchat_buddy_info_t));
		memcpy(&binfo->node, node, sizeof(struct btcclient_protocol_account_t));
		memcpy(binfo->bd_addr, bd_addr, sizeof(binfo->bd_addr));
		b->proto_data = binfo;

		status = 2 | (node->state<<5);
		//printf("buddy new: %d\n", status);
		serv_got_update(gc, b->name, 1, 0, 0, 0, status);
		serv_got_alias(gc, b->name, node->name);
		//printf("buddy new2: %d\n", b->uc);
	}
	
	return(1);
}

/**
 *  @brief create and setup btChat session globals
 *
 *  @returns pointer to new globals
 */
struct btchat_globals_t* btchat_globals_new()
{
	struct btchat_globals_t *new_globals = NULL;
	
	
	/* allocate and init globals */
	new_globals = (struct btchat_globals_t*) malloc(sizeof(struct btchat_globals_t));
	memset(new_globals, 0, sizeof(struct btchat_globals_t));
	
	/* init buffer */
	new_globals->buffer_size = 0xffff;
	new_globals->buffer_pos = 0;

	new_globals->state = BTCHAT_CLIENT_STATE_DISCONNECTED;
	
	return(new_globals);
}

/**
 *  @brief read bytes from btchat server
 *
 *  @param g pointer to btChat session globals
 *
 *  @returns 1 on success and -1 on failure
 */
int btchat_read(struct btchat_globals_t *g)
{
	int read_bytes;
	
	
	#ifdef BTCHAT_DEBUG
	assert(g != NULL);
	#endif
	
	
	read_bytes = cread(g->socket, g->buffer + g->buffer_pos, g->buffer_size - g->buffer_pos);
	
	if (read_bytes == 0) {
		gaim_debug(GAIM_DEBUG_INFO, "btchat", "btchat_read() read_bytes = 0\n");
		return(-1);
	}
			
	g->buffer_pos += read_bytes;

	gaim_debug(GAIM_DEBUG_INFO, "btchat", "btchat_read() read_bytes = %d\n", read_bytes);

	return(1);
}

/**
 *  @brief get next event from read buffer
 *
 *  If no event is available the NULL event will be returned.
 *
 *  @param g pointer to btChat session globals
 *  @param e pointer to pointer for new event
 *
 *  @returns 1
 */
int btchat_event_get(struct btchat_globals_t *g, struct btchat_event_t **e)
{
	struct btcclient_protocol_header_t *he;
	struct btcclient_protocol_msg_t *msg;
	
	
	#ifdef BTCHAT_DEBUG
	assert(g != NULL);
	#endif
	

	/* create event and set time */
	*e = (struct btchat_event_t*) malloc(sizeof(struct btchat_event_t));	
	memset(*e, 0, sizeof(struct btchat_event_t));
	time(&(*e)->when);

	if (g->buffer_pos >= sizeof(struct btcclient_protocol_header_t)) {
		he = (struct btcclient_protocol_header_t*) g->buffer;
		if (g->buffer_pos >= he->length) {
		
			gaim_debug(GAIM_DEBUG_INFO, "btchat", "btchat_event_get() g->buffer_pos >= he->length\n");
			
			/* node/account/device event */
			if (he->pkg_type == BTCCLIENT_PROTOCOL_ACCOUNT) {
				memcpy(&(*e)->src, g->buffer + sizeof(struct btcclient_protocol_header_t), he->length - sizeof(struct btcclient_protocol_header_t));
				(*e)->etype = BTCHAT_EVENT_ACCOUNT;
				
				/* clean up */
				gaim_debug(GAIM_DEBUG_INFO, "btchat", "btchat_event_get() name = %s\n", (*e)->src.name);
				memmove(g->buffer, (g->buffer + he->length), g->buffer_pos - he->length);
				g->buffer_pos -= he->length;
				
				return(1);
			}
			/* message event */
			else if (he->pkg_type == BTCCLIENT_PROTOCOL_MSG) {
				(*e)->etype = BTCHAT_EVENT_MSG;
				(*e)->data.msg.msg_length = he->length - sizeof(struct btcclient_protocol_header_t) - sizeof(struct btcclient_protocol_msg_t);
				(*e)->data.msg.msg = malloc((*e)->data.msg.msg_length + 1);
				memset((*e)->data.msg.msg, 0, (*e)->data.msg.msg_length + 1);
				memcpy((*e)->data.msg.msg, (g->buffer + sizeof(struct btcclient_protocol_header_t) + sizeof(struct btcclient_protocol_msg_t)), (*e)->data.msg.msg_length);
				
				msg = (struct btcclient_protocol_msg_t*) (g->buffer + sizeof(struct btcclient_protocol_header_t));
				
				memcpy((*e)->src.addr, msg->src, sizeof(msg->src));

				/* clean up */
				/*gaim_debug(GAIM_DEBUG_INFO, "btchat", "btchat_event_get() name = %s\n", (*e)->src.name);*/
				memmove(g->buffer, (g->buffer + he->length), g->buffer_pos - he->length);
				g->buffer_pos -= he->length;
				
				return(1);
			}
			/* unknown pkg_type */
			else {
				gaim_debug(GAIM_DEBUG_INFO, "btchat", "btchat_event_get() unknown pkg_type\n");
				
				/* reset buffer */
				g->buffer_pos = 0;
			}
		}
	}

	/* nothing happened - send null event */
	(*e)->etype = BTCHAT_EVENT_NULL;

	return(1);
}

/**
 *  @brief free event memory
 *
 *  The pointer will be zeroed after releasing the memory.
 *
 *  @param e pointer to pointer to event
 *
 *  @returns 1
 */
int btchat_event_free(struct btchat_event_t **e)
{
	struct btchat_event_t *t = *e;
	
	
	switch (t->etype) {
	case BTCHAT_EVENT_MSG:
		if (t->data.msg.msg != NULL) free(t->data.msg.msg);
		break;
	}
	
	free(t);
	*e = NULL;
	
	return(1);
}

/*
 *  $Log$
 */
