/*
 *  $Id$
 */
 
/**
 *  @file btchat_bluetooth_class.c
 *
 *  @project btChat (Collin's btChat) (http://www.mulliner.org/bluetooth/btchat/)
 *
 *  @author Collin R. Mulliner <collin(AT)betaversion.net>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "btchat_bluetooth_class.h"

/**
 *  @brief lookup the text representation of the bluetooth device class
 *
 *  @param class (input) the class (in lower 3 bytes)
 *  @param sep   (input) separator for Service Class output
 *  @param msc   (output) the Major Service Class
 *  @param mjc   (output) the Major Device Class
 *  @param mic   (output) the Minor Device Class
 *
 *  @returns nothing
 */
void bluetooth_classToString(unsigned int class, char *sep, char *msc, char *mjc, char *mic)
{
	/* bit masks for Service Class */
	const struct bluetooth_CoD_MSC_t MSC_Table[] = {
	{1024, "Information"},
	{512, "Telephony"},
	{256, "Audio"},
	{128, "Object Transfer"},
	{64, "Capturing"},
	{32, "Rendering"},
	{16, "Networking"},
	{8, "Positioning"},
	{4, "Reserved"},
	{2, "Reserved"},
	{1, "Limited Discoverable Mode"},
	{0, "0"}};

	/* table for Major Device Class */
	const struct bluetooth_CoD_MJDC_t MJDC_Table[32] = {
	{0, "Miscellaneous", 6},
	{1, "Computer", 6},
	{2, "Phone", 6},
	{3, "LAN/Network Access point", 6},
	{4, "Audio/Video", 6},
	{5, "Peripheral", 2},
	{6, "Imaging", 4},
	{7, "", 6},
	{8, "", 6},
	{9, "", 6},
	{10, "", 6},
	{11, "", 6},
	{12, "", 6},
	{13, "", 6},
	{14, "", 6},
	{15, "", 6},
	{16, "", 6},
	{17, "", 6},
	{18, "", 6},
	{19, "", 6},
	{20, "", 6},
	{21, "", 6},
	{22, "", 6},
	{23, "", 6},
	{24, "", 6},
	{25, "", 6},
	{26, "", 6},
	{27, "", 6},
	{28, "", 6},
	{29, "", 6},
	{30, "", 6},
	{31, "Uncategorized"}};

	/* table for Minor Device Class */
	const struct bluetooth_CoD_MIDC6 MIDC1_Table[64] = {
	{0, "Uncategorized"},
	{1, "Desktop workstation"},
	{2, "Server-class computer"},
	{3, "Laptop"},
	{4, "Handheld PC/PDA"},
	{5, "Palm sized PC/PDA"},		
	{6, "Wearable computer"}};
	
	/* table for Minor Device Class */
	const struct bluetooth_CoD_MIDC6 MIDC2_Table[64] = {
	{0, "Uncategorized"},
	{1, "Cellular"},
	{2, "Cordless"},
	{3, "Smart phone"},
	{4, "Wired modem or voice gateway"},
	{5, "Common ISDN Access"}};
	
	/* table for Minor Device Class */
	const struct bluetooth_CoD_MIDC6 MIDC3_Table[64] = {
	{0<<3, "Fully available"},
	{1<<3, "01 - 17% utilized"},	
	{2<<3, "17 - 33% utilized"},		
	{3<<3, "33 - 50% utilized"},
	{4<<3, "50 - 67% utilized"},
	{5<<3, "67 - 83% utilized"},
	{6<<3, "83 - 99% utilized"},
	{7<<3, "No service available"}};

	/* table for Minor Device Class */	
	const struct bluetooth_CoD_MIDC6 MIDC4_Table[64] = {
	{0, "Uncategorized"},
	{1, "Device conforms to the Headset profile"},
	{2, "Hands-free"},
	{3, "(Reserved)"},
	{4, "Microphone"},	
	{5, "Loudspeaker"},			
	{6, "Headphones"},
	{7, "Portable Audio"},
	{8, "Car audio"},
	{9, "Set-top box"},
	{10, "HiFi Audio Device"},
	{11, "VCR"},
	{12, "Video Camera"},
	{13, "Camcorder"},
	{14, "Video Monitor"},
	{15, "Video Display and Loudspeaker"},
	{16, "Video Conferencing"},
	{17, "(Reserved)"},
	{18, "Gaming/Toy"}};
	
	/* table for Minor Device Class */
	const struct bluetooth_CoD_MIDC2 MIDC5_Table[4] = {
	{0, "(Reserved)"},
	{1, "Keyboard", {
		{0, "Uncategorized device"},
		{1, "Joystick"},
		{2, "Gamepad"},
		{3, "Remote control"},
		{4, "Sensing device"},
		{5, "Digitizer tablet"},
		{6, "Card Reader"}}},
	{2, "Pointing device", {
		{0, "Uncategorized device"},
		{1, "Joystick"},
		{2, "Gamepad"},
		{3, "Remote control"},
		{4, "Sensing device"},
		{5, "Digitizer tablet"},
		{6, "Card Reader"}}},
	{3, "Combo keyboard/pointing device", {
		{0, "Uncategorized device"},
		{1, "Joystick"},
		{2, "Gamepad"},
		{3, "Remote control"},
		{4, "Sensing device"},
		{5, "Digitizer tablet"},
		{6, "Card Reader"}}}
	};

	/* table for Minor Device Class */
	const struct bluetooth_CoD_MIDC4 MIDC6_Table[16] = {
	{0, "(Reserved)"},
	{1<<2, "Display"},
	{2<<2, "Camera"},
	{3<<2, "Display/Camera"},
	{4<<2, "Scanner"},
	{5<<2, "Display/Scanner"},
	{6<<2, "Camera/Scanner"},
	{7<<2, "Display/Camera/Scanner"},
	{8<<2, "Printer"},
	{9<<2, "Display/Printer"},
	{10<<2, "Camera/Printer"},
	{11<<2, "Display/Camera/Printer"},
	{12<<2, "Scanner/Printer"},
	{13<<2, "Display/Scanner/Printer"},
	{14<<2, "Camera/Scanner/Printer"},
	{15<<2, "Display/Camera/Scanner/Printer"}};

	/* variables */
	unsigned int serviceClass;
	unsigned int majorDeviceClass;
	unsigned int minorDeviceClass;
	int i, p;
	
	/* copy to bit field */
	serviceClass = class>>13;
	majorDeviceClass = (class<<19)>>27;
	minorDeviceClass = (class<<24)>>26;
	
	#ifdef DEBUF3
	printf("Service Class: %d, Major: %d, Minor: %d\n", serviceClass,
		majorDeviceClass, minorDeviceClass);
	#endif
	
	if (msc != NULL) {
		/* get Service Class */
		for (p = 0, i = 0; i < 11; i++) {
			if ((serviceClass & MSC_Table[i].mask) == MSC_Table[i].mask) {
				if (p > 0) { sprintf(msc+p, "%s", sep); p+=strlen(sep); }
				sprintf(msc+p, "%s", MSC_Table[i].name);
				p += strlen(MSC_Table[i].name);
			}
		}
	}

	if (mjc != NULL) {
		/* Major Class */
		sprintf(mjc, "%s", MJDC_Table[majorDeviceClass].name);
	}
	
	if (mic != NULL) {
		/* Minor Class */
		switch (majorDeviceClass) {
			case 1:
				sprintf(mic, "%s", MIDC1_Table[minorDeviceClass].name);
				break;
			case 2:
				sprintf(mic, "%s", MIDC2_Table[minorDeviceClass].name);
				break;
			case 3:
				sprintf(mic, "%s", MIDC3_Table[minorDeviceClass].name);
				break;
			case 4:
				sprintf(mic, "%s", MIDC4_Table[minorDeviceClass].name);
				break;
			case 5:
				sprintf(mic, "%s %s", MIDC5_Table[minorDeviceClass>>4].name,
					MIDC5_Table[minorDeviceClass>>4].minor[(minorDeviceClass<<2)>>2].name);
				break;
			case 6:
				sprintf(mic, "%s", MIDC6_Table[(minorDeviceClass>>2)<<2].name);
				break;
			default:
				mic[0] = 0;
				break;
		}
	}
}	

#ifdef BUILD_BT_CLASS_APP
int main(int argc, char **argv)
{
	unsigned char myClass[] = { "502204" }; /* nokia 7650 */
	char mic[1024];
	char mjc[1024];
	char msc[1024];
	unsigned int c;
	

	if (argc > 1) {
		c = strtoul(argv[1], NULL, 16);
	}
	else {
		c = strtoul(myClass, NULL, 16);
		printf("\nsyntax: %s <device class/hcitool output>\n\texample: %s 502204\n\n\n",argv[0],argv[0]);
	}
	
	bluetooth_classToString(c, ", ",msc,mjc,mic);
	printf("Service Class: %s, Major: %s, Minor: %s\n",msc,mjc,mic);
	
	return(1);
}
#endif

/*
 *  $Log$
 */
